<?php
namespace Odhen\API\Service;

use \SimpleXMLElement;

class CartaoTodo {
	protected $sessionHandler;
	
	public function __construct(
		\Helpers\Environment $sessionHandler
	){
        $this->sessionHandler = $sessionHandler;
	}
    
    public function chamadaAPICartaoTodo($rota, $params) {
		self::LogCartaoTodo('');
        self::LogCartaoTodo('chamadaAPICartaoTodo');
		$URL = $this->sessionHandler->getUserInfo()['CDURLINTCARTPRE'];
        $URL = rtrim($URL, '/') . "/" . $rota;
        $curl = curl_init();
        self::LogCartaoTodo('URL: '.$URL);
		curl_setopt($curl, CURLOPT_URL, $URL);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/xml'));

        $XMLparam = self::ArrayToXml($params);
        self::LogCartaoTodo('PARAMETERS: '. preg_replace( '/[\n]/', "", $XMLparam ));
        curl_setopt($curl, CURLOPT_POSTFIELDS,$XMLparam);

        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_ENCODING,'');
        curl_setopt($curl, CURLOPT_MAXREDIRS,10);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_HTTP_VERSION , CURL_HTTP_VERSION_1_1);

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 20);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 20);

        $curl_response = curl_exec($curl);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        self::LogCartaoTodo('HTTPCODE: '.$httpcode);
        if ($httpcode ==='404' || $httpcode === 404){
            self::LogCartaoTodo('Erro 404');
            throw new \Exception("<br>Erro 404. Servidor não encontrado. Verifique se a URL parametrizada esta correta.");
        } else {
            self::LogCartaoTodo('RESPONSE: '.$curl_response);
        }
        if ($err){
            self::LogCartaoTodo('errmsg: '.$errmsg);
            if ($err === 28){
                if(empty($errmsg)){
                    $errmsg = "<br>N&atilde;o foi poss&iacute;vel estabelecer uma conex&atilde;o com o servidor.";
                }
                throw new \Exception($errmsg);
            } else {
                self::LogCartaoTodo('err: '.$err);
                throw new \Exception("<br>Ocorreu um erro na chamada com o servidor:<br><br>" . $errmsg);
            }
        }
        if (empty($curl_response)){
            self::LogCartaoTodo('empty curl response');
            throw new \Exception("<br>N&atilde;o houve retorno do servidor.");
        }
        try{
            $result = self::XmlToJson($curl_response);
            $result = json_decode($result, true);
        } catch(\Exception $e){
            $result = json_decode($curl_response, true); 
            if($result){
                if(isset($result['error'])){
                    $result['success'] = false;    
                }
            }
        }
		if (empty($result)) {
            $result = array('error'=>true,'message'=>$curl_response);
        }
		if (!isset($result['error'])){
			$result['error']=false;
		}
		$result['error'] = ($result['error'] === true || $result['error'] === 'true');

		if (isset($result['response_code'])){
			if ($result['response_code']!='00'){
				$result['error']=true;
			}
		}
		if($result['error']){
            self::LogCartaoTodo('SUCCESS false');
            self::LogCartaoTodo('');
			if(isset($result['message'])) {
				//throw new \Exception('Erro: '.$result['message']);
				return $result;
			} else {
				throw new \Exception("Ocorreu um erro não esperado na consulta ao servidor.".(!empty($rota)?' Endpoint '.$rota.'.':''));
			}
		} else {
			self::LogCartaoTodo('SUCCESS true');
            self::LogCartaoTodo('');
			return $result;            
		}
	}

	public function XmlToJson ($xml) {
        self::LogCartaoTodo('XmlToJson');
        $xml = str_replace(array("\n", "\r", "\t"), '', $xml);
        $xml = trim(str_replace('"', "'", $xml));
        $simpleXml = simplexml_load_string($xml);
        $json = json_encode($simpleXml);
        self::LogCartaoTodo($json);
        return $json;
    }

    function ArrayToXml($array, $xml = null){
        $boprimeira = false;
        if($xml == null){
            $xml = new SimpleXMLElement('<root/>');
            $boprimeira = true;
            self::LogCartaoTodo('ArrayToXml');
        }        
        foreach ($array as $key => $value) {
            if(is_int($key) && is_array($value)){
                self::ArrayToXml($value, $xml);
            } else {
                if(is_int($key)){
                    if($key===0){
                        $key = '0';
                    } else {
                        $key = strval($key);
                    }
                }
                if(is_bool($key)){
                    if($key){
                        $key = 'true';
                    }else {
                        $key = 'false';
                    }
                }
                if(is_array($value)){
                    $label = $xml->addChild($key);
                    self::ArrayToXml($value, $label);
                }
                else {
                    $xml->addChild($key, $value);
                }
            }
        }
        if($boprimeira){
            //self::LogUDS($xml->asXml());
        }
        return $xml->asXml();
    }

    public function LogCartaoTodo($texto){
        return self::Log('CartaoTodo',$texto);
    }

    public function Log($tipo,$texto){
        if($tipo=='CartaoTodo'){
            $tipo ='cartaoTodo';
        }
        $paths = ['Teknisa/','LOG/', $tipo.'/', (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('m-Y')];
        $logVendaPath = 'C:/';
        foreach ($paths as $path){
            $logVendaPath = $logVendaPath . $path;
            if (!is_dir($logVendaPath)){
                mkdir($logVendaPath, 0700);
            }
        }
        $logVendaPath = $logVendaPath .'/'. (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('d-m-Y') . '.txt';
        $horaAtual = (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('d/m/Y H:i:s');
        file_put_contents($logVendaPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
    }
}