<?php
namespace Service;

use \Util\Exception;

class FiscalFunctions {

    protected $entityManager;
	protected $util;
	protected $fiscalSantaCatarina;
	
	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager, 
		\Util\Util $util, 
		\Odhen\API\Service\FiscalSantaCatarina $fiscalSantaCatarina
	){
		$this->entityManager = $entityManager;
		$this->util = $util;
		$this->fiscalSantaCatarina = $fiscalSantaCatarina;
		
	}

	public function prepareCurlObject($timeout) {
		$this->fiscalSantaCatarina->createCurlObject($timeout);
		$this->entityManager->getConnection()->beginTransaction();
		$session = $this->util->getSessionVars(null);
        $params = array (
            'CDFILIAL' => $session['CDFILIAL'],
            'CDCAIXA' => $session['CDCAIXA'],
            'NRORG' => $session['NRORG']
        );
        $DSSATHOST = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_SAT", $params)['DSSATHOST'];
        //if(empty($DSSATHOST)) throw new Exception("Não foi possível realizar a geração do relatório.<br><br> Endereço de acesso ao servidor periféricos se encontra vazio. Verifique a parametrização e tente novamente.");
		$this->fiscalSantaCatarina->setCurlUrl($DSSATHOST);
	}


	public function vendasCPFCNPJ($cdFilial, $cpfCnpj, $periodo, $nmApp, $version){
		try {
			$this->prepareCurlObject(40);
			$result =  $this->fiscalSantaCatarina->vendasCPFCNPJ($cdFilial, $cpfCnpj, $periodo, $nmApp, $version);
		} catch(\Exception $e){
			Exception::logException($e);
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}

		return $result;
	}

	public function registroPaf($cdFilial, $cdCaixa, $dataInicial, $dataFinal, $nmApp){
		try {
			$this->prepareCurlObject(360);
			$result = $this->fiscalSantaCatarina->registroPaf($cdFilial, $cdCaixa, $dataInicial, $dataFinal, $nmApp);
		} catch(\Exception $e){
			Exception::logException($e);
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}

		return $result;
	}

	public function requisicaoExterna($cdFilial, $dataInicial, $dataFinal, $nmApp, $version){
		try {
			$this->prepareCurlObject(40);
			$result = $this->fiscalSantaCatarina->requisicaoExterna($cdFilial, $dataInicial, $dataFinal, $nmApp, $version);
		} catch(\Exception $e){
			Exception::logException($e);
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}

		return $result;
	}

}